(* This program is free software.  See LICENSE for more information *)

open Smt_sorts;;
open Smt_tables;;
open Smt_helper_theory;;
open Smt_parser_theory;;
open Smt_lexer_theory;;
open Smt_helper_logic;;
open Smt_parser_logic;;
open Smt_lexer_logic;;
open Smt_helper;;
open Smt_parser;;
open Smt_lexer;;



(*************** General use functions *********************************)
	  
let print_entry a b = Printf.printf "%s = %s\n" a b;;  
  
let print_table table = Hashtbl.iter print_entry table;;



let parse_benchmark_file filename = 
	let in_ch = open_in filename in
	let lexbuf = Lexing.from_channel in_ch in
	Hashtbl.clear Smt_tables.symbol_table;
	Hashtbl.clear Smt_tables.sorts;
    Hashtbl.clear Smt_tables.funs;
	Hashtbl.clear Smt_tables.preds;
	      
	Hashtbl.replace Smt_tables.symbol_table "---" "---";
	Hashtbl.replace Smt_tables.sorts "---" "---";
	Hashtbl.replace Smt_tables.sorts "_numeralsort_" "UNKNOWN";
	Hashtbl.replace Smt_tables.funs "---" "---";
	Hashtbl.replace Smt_tables.preds "---" "---";

	Printf.printf "\nParsing Benchmark File: %s" filename;
	let benchmark = Smt_parser.benchmark Smt_lexer.token lexbuf in
	close_in in_ch;

	let formula = Smt_helper.get_formula_from_benchmark benchmark in
	let assumption = Smt_helper.get_assumption_from_benchmark benchmark in
	let env_sorts = Hashtbl.find Smt_tables.environment "env" in
(*	let env = if ((List.length formula) = 0) then
			[]
		else
			Smt_helper.check_formula env_sorts (List.hd formula)
	in *)
	(*
	let env = if ((List.length assumption) = 0) then
			[]
		else
			Smt_helper.check_formula env_sorts (List.hd assumption)
	in
*)	
	Hashtbl.clear Smt_tables.environment;
(*	Hashtbl.replace Smt_tables.environment "env" []; *)

	benchmark;;
	
let parse_logic_file filename = 
	let in_ch = open_in filename in
	let lexbuf = Lexing.from_channel in_ch in
	
	Hashtbl.clear Smt_tables.symbol_table;
	Hashtbl.clear Smt_tables.sorts;
    Hashtbl.clear Smt_tables.funs;
	Hashtbl.clear Smt_tables.preds;
	      
	Hashtbl.replace Smt_tables.symbol_table "---" "---";
	Hashtbl.replace Smt_tables.sorts "---" "---";
	Hashtbl.replace Smt_tables.funs "---" "---";
	Hashtbl.replace Smt_tables.preds "---" "---";
	
	Printf.printf "\nParsing Logic File: %s\n" filename;
	let logic = Smt_parser_logic.logic Smt_lexer_logic.token lexbuf in
	close_in in_ch;
	logic;;
	

let parse_theory_file filename = 
	let in_ch = open_in filename in
	let lexbuf = Lexing.from_channel in_ch in
(* 	let env_sorts = [FUNCTION ("select",["Array";"Index"],"Element");FUNCTION("store",["Array";"Index";"Element"],"Array")] in *)

 	Hashtbl.clear Smt_tables.symbol_table;
	Hashtbl.clear Smt_tables.sorts;
	Hashtbl.clear Smt_tables.funs;
	Hashtbl.clear Smt_tables.preds;
	      
	Hashtbl.replace Smt_tables.symbol_table "---" "---";
	Hashtbl.replace Smt_tables.sorts "---" "---";
	Hashtbl.replace Smt_tables.funs "---" "---";
	Hashtbl.replace Smt_tables.preds "---" "---";
	
	
	Printf.printf "Parsing Theory File: %s\n" filename;
	let theory = Smt_parser_theory.theory Smt_lexer_theory.token lexbuf in
	close_in in_ch;
	let axioms = get_axioms_from_theory theory in
	let env_sorts = Hashtbl.find Smt_tables.environment "env" in
	let env = if ((List.length axioms) = 0) then
			[]
		else
			List.map (Smt_helper_theory.check_formula env_sorts) (List.hd axioms)
	in
	
	Hashtbl.clear Smt_tables.environment;
	Hashtbl.replace Smt_tables.environment "env" [];
	theory;;



	
